#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
jinja_kit.exceptions
~~~~~~~~~~~~~~~~~~~~

Base jinja_kit exceptions

:copyright: (c) 2013 by Alexandr Lispython (alex@obout.ru).
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/jinja-kit
"""

from jinja2.exceptions import (
    TemplateError, FilterArgumentError, SecurityError,
    TemplateAssertionError, TemplateError,
    TemplateNotFound, TemplateRuntimeError,
    TemplateSyntaxError, UndefinedError)


class JinjaKitTemplateError(TemplateError):
    """Baseclass for all template errors."""


class JinjaKitTemplateSyntaxError(TemplateSyntaxError):
    """Raised to tell the user that there is a problem with the template."""



# -*- coding: utf-8 -*-
#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
jinja_kit.contrib.django.filters
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Template filters

:copyright: (c) 2013 by Alexandr Lispython (alex@obout.ru).
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/jija-kit
"""

from django import template
from django.conf import settings
from django.utils.safestring import mark_safe
from django.utils.http import urlquote_plus, urlencode, urlquote
from django.utils.encoding import force_unicode
from django.template.defaultfilters import escapejs

from jinja_kit.utils import sanitize_html


def markdown(value, ext=[]):
    """
    Runs Markdown over a given value, optionally using various
    extensions python-markdown2 supports.

    Syntax::

        {{ value|markdown(['extension1_name','extension2_name']) }}

    To enable safe mode, which strips raw HTML and only returns HTML
    generated by actual Markdown syntax, pass "safe" as the first
    extension in the list.

    If the version of Markdown in use does not support extensions,
    they will be silently ignored.

    """
    try:
        import markdown2
    except ImportError:
        if settings.DEBUG:
            raise template.TemplateSyntaxError, "Error in {% markdown %} filter: The Python markdown2 library isn't installed."
        return sanitize_html(force_unicode(value))
    else:
        if len(ext) > 0 and ext[0] == "safe":
            ext = ext[1:]
            safe_mode = True
        else:
            safe_mode = False
        return mark_safe(markdown2.markdown(sanitize_html(force_unicode(value)), extras=ext, safe_mode=safe_mode))


filters = {
    'markdown': markdown,
    'urlquote_plus': urlquote_plus,
    'urlquote': urlquote,
    'urlencode': urlencode,
    'escapejs': escapejs
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from django.template.response import TemplateResponse
from django.views.generic.base import TemplateResponseMixin as DjangoTemplateResponseMixin

from jinja_kit.contrib.django import jinja_kit_django

class JinjaTemplateResponse(TemplateResponse):
    kit = jinja_kit_django

    rendering_attrs = ['template_name', 'context_data', '_post_render_callbacks']

    def __init__(self, request, template, context=None, mimetype=None,
                 status=None, content_type=None, current_app=None, **kwargs):

        self.processors = []

        super(JinjaTemplateResponse, self).__init__(
            request, template, context, mimetype,
            status, content_type, current_app, **kwargs)


    def resolve_template(self, template):
        return self.kit.select_template(template)

    def resolve_context(self, context):
        return context

    @property
    def rendered_content(self):
        return self.kit.render_to_string(template_name=self.template_name,
                                         context=self.resolve_context(self.context_data),
                                         request=self._request, processors=self.processors)


class TemplateResponseMixin(DjangoTemplateResponseMixin):
    response_class = JinjaTemplateResponse

    def render_to_response(self, context, template_name = None, **response_kwargs):
        """
        Returns a response with a template rendered with the given context.
        """
        return self.response_class(
            request=self.request,
            template=template_name or self.get_template_names(),
            context=context,
            **response_kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import absolute_import


from jinja_kit import Kit
import gettext
from django.utils import translation
from django.conf import settings
from django.http import HttpResponse
from django.template.context import get_standard_processors

from jinja2 import Environment, FileSystemLoader
from jinja2.utils import import_string

class DjangoKit(Kit):
    """Magic wrapper for jinja environment for django
    """

    def __init__(self, settings, loader=FileSystemLoader):
        self.settings = settings

        self.search_path = getattr(settings, 'TEMPLATE_DIRS', list())
        self.loader = import_string(getattr(settings, 'TEMPLATE_LOADER',
                                            'jinja2.FileSystemLoader'))

        self.env = Environment(loader=self.loader(self.search_path),
                               auto_reload=getattr(settings, 'JINJA2_AUTO_RELOAD', False),
                               cache_size=getattr(settings, 'JINJA2_CHACHE_SIZE', 50),
                               extensions=getattr(settings, 'JINJA2_EXTENSIONS', ()))
        self.filters = getattr(settings, 'JINJA2_FILTERS', [])
        self.globals = getattr(settings, 'JINJA2_GLOBALS', [])

        # Not good :-(
        for name, f in self.load_filters(self.filters).iteritems():
            self.add_filter(f, name)

        for name, f in self.load_globals(self.globals).iteritems():
            self.add_global(f, name)

        if getattr(settings, 'USE_I18N'):
            try:
                self.env.install_gettext_translations(translation)
            except Exception:
                self.env.install_null_translations()

    def get_standard_processors(self):
        """Get django standart processors
        """
        return get_standard_processors()

    def render_to_string(self, template_name, context=None, request=None,
                         processors=None):
        return super(DjangoKit, self).render_to_string(
            template_name, context=context, processors=processors,
            processor_arg=request)

    def render_to_response(self, template_name, context=None, request=None,
                           processors=None, mimetype=None,
                           response_class=HttpResponse):
        """Return http response
        """
        if 'request' not in context:
            context['request'] = request
        return response_class(self.render_to_string(
            template_name, context=context,
            processors=processors, request=request),
                              mimetype=mimetype)


jinja_kit_django = DjangoKit(settings)



# -*- coding: utf-8 -*-
"""
jinja_kit.core
~~~~~~~~~~~~~~

Core jinja_kit wrapper class

:copyright: (c) 2009 by the Jinja Team.
:copyright: (c) 2012 - 2013 by Alexandr Lispython (alex@obout.ru).
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/jinja-kit
"""

from itertools import chain

try:
    from cProfile import Profile
except ImportError:
    from profile import Profile

from jinja2 import Environment, FileSystemLoader
from jinja2.utils import import_string

from jinja_kit.exceptions import TemplateNotFound


class Kit(object):
    """Magic wrapper for jinja environment
    """
    def __init__(self, search_path, loader=FileSystemLoader,
                 auto_reload=False, cache_size=50, extensions=(),
                 filters={}, globals={}):
        """Construct wrapper
        """
        self.search_path = search_path
        self.env = Environment(loader=loader(self.search_path),
                               auto_reload=auto_reload,
                               cache_size=cache_size,
                               extensions=extensions)
        self.filters = filters
        self.globals = globals

    def filter(self, name=None):
        def wrapper(func):
            return self.add_filter(func, name)
        return wrapper

    def tglobal(self, name=None):
        def wrapper(func):
            return self.add_global(func, name)
        return wrapper

    def add_filter(self, func, name=None):
        """Add filter to environment
        :param func:
        :param name:
        """
        self.env.filters.update({name or func.func_name: func})
        return func

    def add_global(self, func, name=None):
        """Add global function to context

        :param func:
        :param name:
        """
        self.env.globals.update({name or func.func_name: func})
        return func

    def load_filters(self, paths=None):
        """Load filter to environment
        """
        result = {}
        paths = paths or self.filters
        for m in paths:
            f = __import__(m, fromlist=['filters'])
            result.update(f.filters)
        return result

    def load_globals(self, paths=None, key='globals'):
        """Load global function to environment
        """
        result = {}
        paths = paths or self.globals
        for m in paths:
            f = __import__(m, fromlist=[key])
            result.update(f.globals)
            for name, func in f.globals.iteritems():
                self.add_global(func, name)
        return result

    def load_extensions(self, extensions):
        """Load extensions
        """
        for extension in extensions:
            if isinstance(extension, basestring):
                extension = import_string(extension)
            self.add_extension(extension)

    def add_extension(self, ext):
        """Load extension to environment

        :param ext: extension object
        """
        self.env.add_extension(ext)

    def get_template(self, template_name, global_functions={}):
        """Get template
        """
        return self.env.get_template(template_name, globals=global_functions)

    def select_template(self, templates):
        """Search templates
        """
        if isinstance(templates, (list, tuple)):
            for template in templates:
                try:
                    return self.get_template(template,
                                             global_functions=globals())
                except TemplateNotFound:
                    continue
        elif isinstance(templates, (str, unicode)):
            return self.get_template(templates, globals())

        raise TemplateNotFound

    def render_to_string(self, template_name, context=None,
                         processors=None, processor_arg=None):
        """Render template into string

        :param template_name:
        :param context:
        :param processors:
        :param proccessor_arg:
        """
        context = dict(context or {})
        context['processor_arg'] = processor_arg
        standard_processors = self.get_standard_processors()
        for processor in chain(standard_processors,
                               processors or ()):
            if processor_arg:
                context.update(processor(processor_arg))
        return self.select_template(template_name).render(context)

    def get_standard_processors(self):
        return ()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
jinja_kit.contrib.django.globals
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Template functions

:copyright: (c) 2013 by Alexandr Lispython (alex@obout.ru).
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/jinja-kit
"""

from django.conf import settings
from django.utils.safestring import mark_safe

from jinja2 import contextfunction


def _reverse(viewname, args=[], kwargs={}, fail=True):
    from django.core.urlresolvers import reverse, NoReverseMatch

    url = ''
    try:
        url = reverse(viewname, args=args, kwargs=kwargs)
    except NoReverseMatch:
        projectname = settings.SETTINGS_MODULE.split('.')[0]
        try:
            url = reverse(projectname + '.' + viewname, args=args, kwargs=kwargs)
        except NoReverseMatch:
            if fail:
                raise
            else:
                return ''
    return url


@contextfunction
def csrf_token(context):
    csrf_token = context.get('csrf_token', None)
    if csrf_token:
        if csrf_token == 'NOTPROVIDED':
            return mark_safe(u"")
        else:
            return mark_safe(u"<div style='display:none'><input type='hidden' name='csrfmiddlewaretoken' value='%s' /></div>" % (csrf_token))
    else:
        # It's very probable that the token is missing because of
        # misconfiguration, so we raise a warning
        from django.conf import settings
        if settings.DEBUG:
            import warnings
            warnings.warn("A {% csrf_token %} was used in a template, but the context did not provide the value.  This is usually caused by not using RequestContext.")
            return u''


globals = {
    "url": _reverse,
    "dir": dir,
    'csrf_token': csrf_token
}



# -*- coding: utf-8 -*-
#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
jinja_kit.contrib.django.decorators
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Decorators helpers

:copyright: (c) 2013 by Alexandr Lispython (alex@obout.ru).
:license: BSD, see LICENSE for more details.
:github: http://github.com/Lispython/jinja-kit
"""

from jinja_kit.contrib.django import jinja_kit_django


def render_to(template):
    def renderer(func):
        def wrapper(request, *args, **kw):
            output = func(request, *args, **kw)
            if isinstance(output, (list, tuple)):
                return jinja_kit_django.render_to_response(output[1], output[0], request)
            elif isinstance(output, dict):
                return jinja_kit_django.render_to_response(template, output, request)
            return output
        return wrapper
    return renderer
